package org.nrg.xnat.tags;

import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.LinkedHashSet;
import java.util.regex.Pattern;


/**
 * Tag label class: all instances are normalized, valid tag labels.
 * @author Kevin A. Archie <karchie@npg.wustl.edu>
 */
public final class TagLabel {
    private static final Pattern VALID_LABEL_PATTERN = Pattern.compile("\\w+");
    private final String label;
    private final String norm;
    
    public TagLabel(final String name) throws InvalidTagLabelException {
	if (null == name || !VALID_LABEL_PATTERN.matcher(name).matches()) {
	    throw new InvalidTagLabelException(name);
	}
	this.label = name;
	this.norm = name.toLowerCase();
    }
    
    public int hashCode() {
        return norm.hashCode();
    }
    
    public boolean equals(Object o) {
        return o instanceof TagLabel && norm.equals(((TagLabel)o).norm);
    }
    
    public String toString() { return label; }
    
    @SuppressWarnings("unchecked")
    public static Collection<TagLabel> makeLabels(final Collection names, final Collection<String> invalid) {
        return makeLabels(Collections.enumeration(names), invalid);
    }
    
    @SuppressWarnings("unchecked")
    public static Collection<TagLabel> makeLabels(final Enumeration ne, final Collection<String> invalid) {
	final Collection<TagLabel> r = new LinkedHashSet<TagLabel>();
	while (ne.hasMoreElements()) {
	    final Object o = ne.nextElement();
	    if (null == o) {
		invalid.add(null);
		continue;
	    }
	    final String name = o.toString();
	    try {
		r.add(new TagLabel(name));
	    } catch (InvalidTagLabelException e) {
		invalid.add(name);
	    }
	}
	return r;
    }
}